@@ -1,7 +1,13 @@
 import pytest
+
 from PIL import Image, SgiImagePlugin
 
-from .helper import assert_image_equal, assert_image_similar, hopper
+from .helper import (
+    assert_image_equal,
+    assert_image_equal_tofile,
+    assert_image_similar,
+    hopper,
+)
 
 
 def test_rgb():
@@ -15,10 +21,7 @@ def test_rgb():
 
 
 def test_rgb16():
-    test_file = "Tests/images/hopper16.rgb"
-
-    with Image.open(test_file) as im:
-        assert_image_equal(im, hopper())
+    assert_image_equal_tofile(hopper(), "Tests/images/hopper16.rgb")
 
 
 def test_l():
@@ -37,8 +40,7 @@ def test_rgba():
     test_file = "Tests/images/transparent.sgi"
 
     with Image.open(test_file) as im:
-        with Image.open("Tests/images/transparent.png") as target:
-            assert_image_equal(im, target)
+        assert_image_equal_tofile(im, "Tests/images/transparent.png")
         assert im.get_format_mimetype() == "image/sgi"
 
 
@@ -48,16 +50,14 @@ def test_rle():
     test_file = "Tests/images/hopper.sgi"
 
     with Image.open(test_file) as im:
-        with Image.open("Tests/images/hopper.rgb") as target:
-            assert_image_equal(im, target)
+        assert_image_equal_tofile(im, "Tests/images/hopper.rgb")
 
 
 def test_rle16():
     test_file = "Tests/images/tv16.sgi"
 
     with Image.open(test_file) as im:
-        with Image.open("Tests/images/tv.rgb") as target:
-            assert_image_equal(im, target)
+        assert_image_equal_tofile(im, "Tests/images/tv.rgb")
 
 
 def test_invalid_file():
@@ -71,8 +71,7 @@ def test_write(tmp_path):
     def roundtrip(img):
         out = str(tmp_path / "temp.sgi")
         img.save(out, format="sgi")
-        with Image.open(out) as reloaded:
-            assert_image_equal(img, reloaded)
+        assert_image_equal_tofile(img, out)
 
     for mode in ("L", "RGB", "RGBA"):
         roundtrip(hopper(mode))
@@ -88,8 +87,7 @@ def test_write16(tmp_path):
         out = str(tmp_path / "temp.sgi")
         im.save(out, format="sgi", bpc=2)
 
-        with Image.open(out) as reloaded:
-            assert_image_equal(im, reloaded)
+        assert_image_equal_tofile(im, out)
 
 
 def test_unsupported_mode(tmp_path):
