@@ -1,12 +1,9 @@
 import sys
 
 import pytest
+
 from PIL import Image
 
-from .helper import is_win32
-
-pytestmark = pytest.mark.skipif(is_win32(), reason="Win32 does not call map_buffer")
-
 
 def test_overflow():
     # There is the potential to overflow comparisons in map.c
@@ -20,12 +17,19 @@ def test_overflow():
 
     # This image hits the offset test.
     with Image.open("Tests/images/l2rgb_read.bmp") as im:
-        with pytest.raises((ValueError, MemoryError, IOError)):
+        with pytest.raises((ValueError, MemoryError, OSError)):
             im.load()
 
     Image.MAX_IMAGE_PIXELS = max_pixels
 
 
+def test_tobytes():
+    # Previously raised an access violation on Windows
+    with Image.open("Tests/images/l2rgb_read.bmp") as im:
+        with pytest.raises((ValueError, MemoryError, OSError)):
+            im.tobytes()
+
+
 @pytest.mark.skipif(sys.maxsize <= 2 ** 32, reason="Requires 64-bit system")
 def test_ysize():
     numpy = pytest.importorskip("numpy", reason="NumPy not installed")
